<?php

/*

 $Id: option.php,v 1.20 2004/02/18 23:11:37 vollmerk Exp $
 $Source: /tmp/maintain/modules/class/option.php,v $

 Copyright (c) 2002 Oregon State University - Network Engineering
 All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
*/
/*!
	@header Option Class
	The option class works from the object_option table and determins all of
	the dhcp options on hosts,workgroups and subnets.

*/

class Option {
	var $id;
	var $dhcp_option;
	var $dhcp_option_name;
	var $scope;
	var $value;
	var $type;
	var $object_id;

	// we can populate this stuff from the database -- no sense in
	//   passing it in.
	function Option ($option_id=0) {
		global $conf;

		$this->id   = $option_id;
		
		if ($this->id) { 
			$info = $this->get_info();
			$this->dhcp_option      = $info->dhcp_option;
			$this->dhcp_option_name = $this->get_option_name($this->dhcp_option);
			$this->value            = $info->value;
			$this->type		= $info->type;
			$this->scope		= $info->scope;
			$this->object_id	= $info->object_id;

		} // if this->id
		
	} // Option

	function get_info () {
	
		$sql = "SELECT * FROM object_option WHERE id='$this->id'";
		$db_result = mysql_query($sql, dbh());		

		return mysql_fetch_object($db_result);

	} // get_info

	// return a VLAN object for this subnet
	function get_option_name ($option) {

		$sql = "SELECT * FROM dhcp_options WHERE id='$option'";
		$db_result = mysql_query($sql, dbh());
		$obj = mysql_fetch_object($db_result);

		return $obj->name;

	} // get_option_name

	/*!
		@function has_access
		@discussion this checks to see if the user actually is allowed
			to make updates to this dhcp option, the call made is 
			dependent upon what type of object we are dealing with
		@param username 	the username of the dude trying to mess with stuff
	*/
	function has_access($username) {
		global $user;

		if ($user->is_admin) { 
			return true;
		}
		elseif ($this->type == 'workgroup') { 
			$object = new Workgroup($this->object_id);
			return $object->has_access($username);
		}
		elseif ($this->type == 'host') { 
			$object = new Host($this->object_id);
			return $object->has_access($username);
		}
		
		// If everything falls through then return false as a safety measure
		return false;
		
	} // has_access
	

	/*!
		@function check_option_value
		@discussion this is a large function that based on the dhcp option picked 
			does checks to make sure they entered a valid value. So you can't enter 
			a IP address for the lease-time and a invalid IP for the routers
	*/
	function check_option_value($dhcp_option_id, $value) { 
		global $conf;

		$dhcp_option = $this->get_option_name($dhcp_option_id);

		$ip = new IP(0);

		if (!$value) {
			$conf['error']->add_error('option_value',"Please specificy a value");
		}

		if (preg_match("/\"/", $value)) { 
			$conf['error']->add_error('option_value',"Invalid Value, double quotes are not allowed. They will be added as needed at build time");
		}

		switch ($dhcp_option) { 
			case 'always-reply-rfc1048':
				if (!preg_match("/^on$/i",$value) && !preg_match("/^off$/i",$value)) {
					$conf['error']->add_error('option_value',"Invalid Value $dhcp_option must be On or Off");
				}
				break;
			case 'netbios-node-type':
				if ($value != 1 && $value != 2 && $value != 4 && $value != 8) {
					$conf['error']->add_error('option_value',"Invalid Value $dhcp_option must be 1, 2, 4 or 8");
				}
			case 'default-ip-ttl':
			case 'default-tcp-ttl':
			case 'time-offset':
			case 'min-lease-time':
			case 'max-lease-time':
			case 'default-lease-time':
				if (!is_numeric($value)) { 
					$conf['error']->add_error('option_value',"Invalid Value $dhcp_option must only contain 0-9");
				}
				break;
			case 'domain-name-servers':
			case 'broadcast-address':
			case 'routers':
                                $data = explode(",", $value);
                                foreach ($data as $ip_addr) {
					if (preg_match("/\%[hi]{1}/",trim($ip_addr))) { $valid = true; }
                                        if (!$ip->is_dotted_quad(trim($ip_addr)) && !$valid) {
                                                $conf['error']->add_error('option_value', "Invalid Value, $dhcp_option format must be 10.0.0.0,10.0.0.1");
                                        }
					unset($valid);
                                }
                                break;
			case 'ntp-servers':
			case 'swap-server':
			case 'nisplus-servers':
			case 'nntp-server':
			case 'pop-server':
			case 'resource-location-servers':
			case 'root-path':
			case 'smtp-server':
			case 'nds-servers':
			case 'cookie-servers':
			case 'netbios-name-servers':
			case 'netbios-dd-server':
			case 'www-server':
			case 'x-display-manager':
			case 'time-servers':
			case 'finger-server':
			case 'font-servers':
			case 'nis-servers':
			case 'impress-servers':
			case 'irc-server':
			case 'lpr-servers':
			case 'log-servers':
				$data = explode(",", $value);
				foreach ($data as $ip_addr) { 
					if (preg_match("/\%[hi]{1}/",trim($ip_addr))) { $valid = true; }
					if (!$ip->is_dotted_quad(trim($ip_addr)) && !$conf['ui']->is_domain($value) && !$valid) { 
						$conf['error']->add_error('option_value', "Invalid Value, $dhcp_option format must be 10.0.0.0,10.0.0.1 or my.domain,my.domain");
					}
					unset($valid);
				}
				break;
			case 'subnet-mask':
                                $data = explode(",", $value);
				$subnet_values = array ('255','254','252','248','240','224','192','128','0');
                                foreach ($data as $ip_addr) {
					$quads = explode(".",$ip_addr);
					foreach($quads as $quad_value) {
						if (!in_array($quad_value,$subnet_values)) {
							$conf['error']->add_error('option_value',"Invalid Netmask valid values are 255,254,252,248,240,224,192,128,0");
						}
					} // end foreach quads

                                        if (!$ip->is_dotted_quad(trim($ip_addr)) ) {
                                                $conf['error']->add_error('option_value', "Invalid Value, $dhcp_option format must be 255.255.255.0");
                                        }
                                }
                                break;
			case 'domain-name':
				if (preg_match("/\%[h]{1}/",trim($ip_addr))) { $valid = true; }
				if (!$conf['ui']->is_domain($value) && !$valid) {
					$conf['error']->add_error('option_value',"Invalid Domain.");
				}
				break;
			case 'nds-context':
			case 'nds-tree-name':
			case 'nisplus-domain':
				if (!preg_match("/^[\w\d\s\.\-]+$/",$value) ) {
					$conf['error']->add_error('option_value',"Invalid String");
				}
				break;
			case 'static-routes':
                                $data = explode(",", $value);
                                foreach ($data as $ip_route) {
					if (preg_match("/^\%[him]{1}$/",trim($ip_addr))) { $valid = true; }
                                        if (!preg_match("/^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\s\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1.3}$/", $ip_route) && !$valid) {
                                                $conf['error']->add_error('option_value', "Invalid Value, $dhcp_option format should be 10.0.0.1 10.0.1.1, 10.0.0.20 10.0.0.23");
                                        }
					unset($valid);
                                }
                                break;
                        case 'dhcp-parameter-request-list':
                                if (!is_int($value)) {
                                        $conf['error']->add_error('option_value',"Invalid Value, $dhcp_option format must be an int");
                                }

                                break;



		} // end switch on dhcp_option

	} // check_option_value

	/*!
		@function check_scope
		@discussion this checks to make sure you didn't try
			to put a scope on a type that doesn't work
	*/
	function check_scope($scope,$type) {
		
		if ($type === 'host' && $scope != 'host') {
			return false;
		}
		elseif ($type === 'subnet' && $scope ==! 'subnet' ) {
			return false;
		}
		elseif ($type === 'workgroup' && ( $scope ==! 'workgroup' && $scope ==! 'host' ) ) {
			return false;
		}

		return true;


	} // check_scope

	function update_value($new_value) {
		
		$this->update_item('value',$new_value);
		$this->value = $new_value;

	} // update_value

	function update_option($new_option) {
	
		$this->update_item('dhcp_option',$new_option);
		$this->dhcp_option_name = $this->get_option_name($new_option);
		$this->dhcp_option = $new_option;
	} // update_option

	function update_scope($new_scope) {

		$this->update_item('scope',$new_scope);
		$this->scope = $new_scope;

	} // update_scope

	function update_item($field,$value) {

		$sql = "UPDATE object_option SET $field='$value' WHERE id='$this->id'";
		$db_results = mysql_query($sql, dbh());

	} 

	/*!
		@function create_option
		@discussion creates an option in the object_option table
	*/
	function create_option($object_id,$type,$scope,$option,$value) {
	
		$sql = "INSERT INTO object_option (object_id,type,scope,dhcp_option,value) VALUES ('$object_id','$type','$scope','$option','$value')";
		$db_results = mysql_query($sql, dbh());
		$id = mysql_insert_id(dbh());

		return $id;

	} // create_option

	function delete () {
		global $conf,$user;

		// Delete the option
		$sql = "DELETE FROM object_option WHERE id=$this->id";
		$db_result = mysql_query($sql, $conf['dbh']);

		// Ugg do a lot of work to get the information so we can add deletion history
		$text = "Deleted " . $this->type . " DHCP Option (" . $this->dhcp_option_name . ") with a" .
			" value of " . $this->value . " and a scope of " . $this->scope;
		$conf['ui']->add_delete_history(0,$text,$user->username);
		
	} //delete

} // class Option

?>
